# Licensed under a 3-clause BSD style license - see LICENSE.rst
# -*- coding: utf-8 -*-
def readcol(name,**kwargs):
    """This is a placeholder for the real Goddard readcol
    function.  Eventually this will match the readcol functionality.
    """
    from ...pydlutils.misc import djs_readcol
    return djs_readcol(name,**kwargs)
