# Licensed under a 3-clause BSD style license - see LICENSE.rst
# -*- coding: utf-8 -*-
def test_findspec():
    """This is just a placeholder for now."""
    from .. import findspec
    #slist = findspec(infile='file.in',sdss=True)
    #if slist is not None:
    #    print slist
    assert True
    return
