# Licensed under a 3-clause BSD style license - see LICENSE.rst
# -*- coding: utf-8 -*-
"""
This subpackage corresponds to the yanny directory of idlutils.
"""
from .write_ndarray_to_yanny import write_ndarray_to_yanny
from .yanny import yanny
