# Licensed under a 3-clause BSD style license - see LICENSE.rst
# -*- coding: utf-8 -*-
"""
This subpackage corresponds to the image directory of idlutils.
"""
from .djs_maskinterp import djs_maskinterp
