# Autogenerated by Astropy-affiliated package pydl's setup.py on 2014-05-06 18:31:40.713347

from astropy.version_helpers import update_git_devstr, get_git_devstr

_last_generated_version = '0.2.2'

version = update_git_devstr(_last_generated_version)
githash = get_git_devstr(sha=True, show_warning=False)

major = 0
minor = 2
bugfix = 2

release = True
debug = False

try:
    from .utils._compiler import compiler
except ImportError:
    compiler = "unknown"

try:
    from .cython_version import cython_version
except ImportError:
    cython_version = "unknown"
