# Licensed under a 3-clause BSD style license - see LICENSE.rst
# -*- coding: utf-8 -*-
"""
This subpackage corresponds to the cooling directory of idlutils.
"""
from .read_ds_cooling import read_ds_cooling
